/*--------------------------------------------------------------------------
 *  Copyright 2008 utgenome.org
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// utgb-shell Project
//
// Create.java
// Since: Jan 9, 2008
//
// $URL$ 
// $Author$
//--------------------------------------
package org.utgenome.shell;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.Properties;

import org.xerial.util.FileResource;
import org.xerial.util.ResourceFilter;
import org.xerial.util.StringUtil;
import org.xerial.util.cui.OptionHandler;
import org.xerial.util.cui.OptionParser;
import org.xerial.util.cui.OptionParserException;
import org.xerial.util.io.VirtualFile;
import org.xerial.util.log.Logger;
import org.xerial.util.template.Template;

/**
 * Create sub command
 * 
 * @author leo
 * 
 */
public class Create extends UTGBShellSubCommandBase {

	private static Logger _logger = Logger.getLogger(Create.class);

	public static enum Opt {
		PROGRAM_LANGUAGE, OUTPUT_FOLDER, PACKAGE_NAME, GROUP
	}

	private OptionParser<Opt> optionParser = new OptionParser<Opt>();

	private String lang = "java";
	private String outputFolder = "";
	private String packageName = null;
	private final String actionPackageName = "app"; // fixed
	private String projectName = "default";
	private String group = "org.utgenome.track";

	private Properties property = new Properties();

	public Create() {

		// setup options
		/*
		 * optionParser.addOptionWithArgument(Opt.PROGRAM_LANGUAGE, "l", "lang", "PROGRAM_TYPE", "output project type
		 * (java, ruby). default=java", new OptionHandler<Opt>() { public void handle(OptionParser<Opt> parser) throws
		 * OptionParserException { lang = parser.getValue(Opt.PROGRAM_LANGUAGE); } });
		 */
		optionParser.addOptionWithArgument(Opt.OUTPUT_FOLDER, "o", "out", "FOLDER", "project output folder (default is the current directory)",
				new OptionHandler<Opt>() {
					public void handle(OptionParser<Opt> parser) throws OptionParserException {
						outputFolder = parser.getValue(Opt.OUTPUT_FOLDER);
					}
				});
		optionParser.addOptionWithArgument(Opt.PACKAGE_NAME, "p", "package", "PACKAGE_NAME", "specify the package name, e.g. org.yourdomain.track",
				new OptionHandler<Opt>() {
					public void handle(OptionParser<Opt> parser) throws OptionParserException {
						packageName = parser.getValue(Opt.PACKAGE_NAME);
					}
				});

		optionParser.addOptionWithArgument(Opt.GROUP, "g", "group", "GROUP_NAME", "specify the maven group name of this project. default = org.utgenome.track",
				new OptionHandler<Opt>() {
					public void handle(OptionParser<Opt> parser) throws OptionParserException {
						group = parser.getValue(Opt.GROUP);
					}
				});
	}

	public void execute(String[] args) throws UTGBShellException {
		try {
			optionParser.parse(args);

			if (optionParser.getArgumentLength() <= 0) {
				System.err.println("no project name is given.");
				return;
			}
			else
				projectName = optionParser.getArgument(0);

			for (int i = 0; i < projectName.length(); i++) {
				if (StringUtil.isWhiteSpace(projectName.substring(i, i + 1))) {
					System.err.println("project name must not contain white spaces: " + projectName);
					return;
				}
			}

			if (packageName == null)
				packageName = projectName;

			property.put("projectName", projectName);
			property.put("package", packageName);
			property.put("group", group);
			property.put("explodedWebappDir", EXPLODED_WEBAPP_DIR);

			for (Object keyObj : property.keySet()) {
				String key = keyObj.toString();
				_logger.info(key + "\t = " + property.getProperty(key));
			}

			if (outputFolder.length() <= 0)
				outputFolder = projectName;
			else
				outputFolder = outputFolder + "/" + projectName;

			_logger.info("project folder: " + outputFolder);
			// compute class path entries for the Eclipse project
			String classPathEntries = computeClassPathEntries();
			property.put("classpathEntry", classPathEntries);

			copyScaffold(outputFolder);

			createStatupCode(outputFolder, packageName);

		}
		catch (Exception e) {
			throw new UTGBShellException(e);
		}
	}

	private void createStatupCode(String projectFolder, String packageName) throws UTGBShellException {
		String packageSourceCodeFolder = projectFolder + "/src/main/java/" + packageName.replaceAll("\\.", "/");

		createFileFromTemplate(UTGBShell.class, "template/java/TrackStartup.java.template", packageSourceCodeFolder + "/TrackStartup.java", property);

	}

	private String computeClassPathEntries() {

		StringBuilder builder = new StringBuilder();
		builder.append(computeClassPathEntries("org.utgenome.shell.template.java.scaffold.src.main.webapp.WEB-INF.lib", "src/main/webapp/WEB-INF/lib"));
		builder.append(computeClassPathEntries("org.utgenome.shell.template.java.scaffold.lib", "lib"));
		return builder.toString();
	}

	private String computeClassPathEntries(String scaffoldPackage, String libFolder) {
		if (!libFolder.endsWith("/"))
			libFolder = libFolder + "/";

		StringBuilder builder = new StringBuilder();

		List<VirtualFile> libResources = FileResource.listResources(scaffoldPackage, new ResourceFilter() {
			public boolean accept(String resourcePath) {
				return resourcePath.endsWith("jar");
			}
		});
		for (VirtualFile vf : libResources) {
			builder.append("<classpathentry kind=\"lib\" path=\"");
			builder.append(libFolder + vf.getLogicalPath());
			builder.append("\"/>\n");
		}
		return builder.toString();

	}

	@Override
	public String name() {
		return "create";
	}

	public String getDetailedDescription() {
		return loadUsage("help-create.txt");
	}

	public String getOneLinerDescription() {
		return "create a new Maven project for implmenting your own track.";
	}

	public String getOptionList() {
		return optionParser.helpMessage();
	}

	/**
	 * Creates the folder structure for the tomcat
	 * 
	 * @param catalinaBase
	 * @throws IOException
	 */
	private void copyScaffold(String outputDir) throws IOException {
		// create the base folder for the scaffold
		File outputFolder = new File(outputDir);

		List<VirtualFile> scaffoldResources = FileResource.listResources("org.utgenome.shell.template.java.scaffold");

		if (scaffoldResources.size() <= 0)
			throw new IllegalStateException("org.utenome.shell.template.java.scaffold is not found");

		// sync scaffoldDir with the output folder
		for (VirtualFile vf : scaffoldResources) {
			String srcLogicalPath = vf.getLogicalPath();

			if (vf.isDirectory()) {

				File dir = new File(outputFolder, srcLogicalPath);
				if (!dir.exists()) {
					_logger.info("create a directory: " + dir.getPath().replaceAll("\\\\", "/"));
					dir.mkdirs();
				}

			}
			else {

				boolean isTemplate = false;
				String outputFileName = srcLogicalPath;

				// is template?
				int extIndex = srcLogicalPath.lastIndexOf(".");
				if (extIndex > 0) {
					String fileExt = srcLogicalPath.substring(extIndex + 1);
					if (fileExt.equals("template")) {
						isTemplate = true;
						outputFileName = srcLogicalPath.substring(0, extIndex);
					}
				}

				File targetFile = new File(outputFolder, outputFileName);
				File parentFolder = targetFile.getParentFile();
				parentFolder.mkdirs();

				// copy the file content without overwrite
				if (!targetFile.exists()) {
					_logger.info("create a file: " + targetFile.getPath().replaceAll("\\\\", "/"));
					InputStream reader = null;
					if (isTemplate) {
						// fill variables in the template
						InputStream templateReader = vf.getURL().openStream();
						Template template = new Template(templateReader);
						String result = template.apply(property);
						reader = new ByteArrayInputStream(result.getBytes());
					}
					else
						reader = vf.getURL().openStream();

					FileOutputStream writer = new FileOutputStream(targetFile);
					byte[] buffer = new byte[1024];
					int bytesRead = 0;
					while ((bytesRead = reader.read(buffer)) > 0) {
						writer.write(buffer, 0, bytesRead);
					}
				}
			}
		}

	}
}
