/*--------------------------------------------------------------------------
 *  Copyright 2008 utgenome.org
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// utgb-shell Project
//
// Server.java
// Since: Jan 10, 2008
//
// $URL$ 
// $Author$
//--------------------------------------
package org.utgenome.shell;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;

import org.utgenome.portable.UTGBServer;
import org.utgenome.shell.config.TrackConfig;
import org.xerial.util.cui.OptionHandler;
import org.xerial.util.cui.OptionParser;
import org.xerial.util.cui.OptionParserException;
import org.xerial.util.log.Logger;

/**
 * A UTGB sub command for starting up a UTGB portable server
 * 
 * @author leo
 * 
 */
public class Server extends UTGBShellSubCommandBase {

	private static Logger _logger = Logger.getLogger(Server.class);

	public enum Opt {
		PORT, GUI
	}

	private OptionParser<Opt> optionParser = new OptionParser<Opt>();

	private int port = 8989;
	private boolean isGUIMode = false;

	public Server() {
		optionParser.addOptionWithArgument(Opt.PORT, "p", "port", "PORT_NUMBER", "port number of the web server. default=8989", new OptionHandler<Opt>() {
			public void handle(OptionParser<Opt> parser) throws OptionParserException {
				port = parser.getIntValue(Opt.PORT);
			}
		});

		optionParser.addOption(Opt.GUI, "g", "gui", "launch in the GUI mode", new OptionHandler<Opt>() {
			public void handle(OptionParser<Opt> parser) throws OptionParserException {
				isGUIMode = true;
			}
		});

	}

	@Override
	public void execute(String[] args) throws UTGBShellException {

		if (!isInProjectRoot())
			throw new UTGBShellException("not in the track project root");

		try {
			optionParser.parse(args);

			ArrayList<String> serverArgs = new ArrayList<String>();
			serverArgs.add("-p");
			serverArgs.add(Integer.toString(port));
			if (isGUIMode)
				serverArgs.add("--gui");
			else
				serverArgs.add("--cui");

			TrackConfig config = loadTrackConfig();
			String projectName = config.getProjectName();
			serverArgs.add("--path=/" + projectName);

			// web app dir
			serverArgs.add("-w");
			serverArgs.add(EXPLODED_WEBAPP_DIR);

			_logger.info("synchronizing the folder contents from" + WEBAPP_FOLDER + " to " + EXPLODED_WEBAPP_DIR);
			rsync(WEBAPP_FOLDER, EXPLODED_WEBAPP_DIR);

			_logger.info(serverArgs);

			UTGBServer server = new UTGBServer(serverArgs.toArray(new String[serverArgs.size()]));
			server.start();

		}
		catch (OptionParserException e) {
			throw new UTGBShellException(e);
		}
		catch (IOException e) {
			throw new UTGBShellException(e);
		}
	}

	public static String getPath(File f) {
		return f.getPath().replaceAll("\\\\", "/");
	}

	public static void rsync(File src, File dest) throws UTGBShellException, IOException {
		if (!src.exists())
			return;

		if (src.isDirectory()) {
			// prepare the target directory
			if (dest.exists()) {
				if (!dest.isDirectory())
					throw new UTGBShellException("cannot create a directory " + getPath(dest) + ": a file with the same name already exists.");
			}
			else {
				_logger.info("create a directory: " + getPath(dest));
				dest.mkdirs();
			}

			// copy contents under the direcotry
			for (File childFile : src.listFiles()) {
				rsync(childFile, new File(dest, childFile.getName()));
			}
		}
		else {
			// copy the src file
			boolean toOverwrite = dest.exists() ? src.lastModified() > dest.lastModified() : true;
			if (!toOverwrite)
				return;

			// prepare the parent folder
			File parentFolder = dest.getParentFile();
			if (!parentFolder.exists()) {
				_logger.info("create a directory: " + getPath(parentFolder));
				parentFolder.mkdirs();
			}
			_logger.info("create a file: " + getPath(dest));
			BufferedInputStream in = new BufferedInputStream(new FileInputStream(src));
			FileOutputStream out = new FileOutputStream(dest);
			byte[] buffer = new byte[1024];
			int bytesRead = 0;
			while ((bytesRead = in.read(buffer)) > 0) {
				out.write(buffer, 0, bytesRead);
			}
			out.flush();
			out.close();
		}

	}

	public static void rsync(String src, String dest) throws UTGBShellException, IOException {
		rsync(new File(src), new File(dest));
	}

	@Override
	public String name() {
		return "server";
	}

	public String getDetailedDescription() {
		return loadUsage("help-server.txt");
	}

	public String getOneLinerDescription() {
		return "start up the portable web server";
	}

	public String getOptionList() {
		return optionParser.helpMessage();
	}

}
